/** 
 * Written by:
 *		Jun Peng (junpeng@stanford.edu)
 *		Kincho Law (law@stanford.edu)
 * ----------------------------------
 * Define all the basic funcations to access data saved in 
 * the OWL (Web Ontology Language) files.
 */ 


import java.io.*;
import java.text.*;
import java.util.*;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;


public class OWLAccess
{
	ResourceBundle rb = ResourceBundle.getBundle("LocalStrings");
    
	private String inputFileName = MyConsts.inputFileName;
	private String myBase = MyConsts.myBase;
	private String fileBase = MyConsts.fileBase;

	private OntModel myModel;

	/**
	 * A constructor that loads the OWL model by reading owl files.
	 */
	public OWLAccess()
	{
		myModel = loadModel();
	}

	public OntModel getOntModel()
	{
		return myModel;
	}


	/* Display the URI to the file on a brower.
	 */
	public void retrieveFile(OntModel theModel, String source, PrintWriter out)
	{
		Vector fileNameV = getInstanceProperty(theModel, source, "name");
		Vector fileURIV = getInstanceProperty(theModel, source, "URI");

		String fileName = getFirstPart((fileNameV.elementAt(0)).toString());
		String fileURI = getFirstPart((fileURIV.elementAt(0)).toString());
        
		out.println("<a href=\"" + fileBase + fileURI + "\">" + fileName + "</a> <br>");
	}


	/* Write all the properties of an Instance (object).
	 * Output to a PrintWriter and pipe to a file later on.
	 */  
	public void writeAnInstance(OntModel theModel, String theInstance, String outFile) 
	{
		try 
		{
			FileWriter out = new FileWriter(outFile);
		
			out.write("<b>The following is the details of " + theInstance + ":</b> <p>");
			Vector allResults = getInstanceAllProperties(theModel, theInstance);

			if (allResults.size() == 0) 
			{
				out.write("The object " + theInstance + " doesn't exist!");
			}

			out.write("<table border=1>");
			out.write("<tr> <td><b>SLOT</b></td>\n" +
				"     <td><b>VALUE</b></td> </tr>");
        
			for (int i = 0; i < allResults.size(); i++)
			{
				OneProperty oneEle = (OneProperty)allResults.elementAt(i);
				out.write("<tr> <td>" + oneEle.name + "</td>");
		
				Vector allValues = oneEle.allValues;
				out.write(" <td>");
				for (int jj = 0; jj < allValues.size(); jj++) 
				{
					if (oneEle.name.startsWith("URI")) 
					{
						String fileLocation = getFirstPart(allValues.elementAt(jj).toString());
						out.write("<a href=\"" + fileBase + fileLocation + "\">" + fileLocation + "</a>");
					}
					else if (oneEle.type.endsWith("string")) 
					{
						out.write(getFirstPart(allValues.elementAt(jj).toString()) + "<br>");
					}
					else if (oneEle.type.startsWith(myBase)) 
					{
						String objName = getShortName(allValues.elementAt(jj).toString());
						out.write("<form method=\"POST\">\n" +
							"<input type=submit name=\"neesobject\" value=\"" + objName + "\">\n" +
							"</form>");
					}
					else 
					{
						out.write(getFirstPart(allValues.elementAt(jj).toString()) + "<br>");
					}
				}
				out.write("</td> </tr>");
			}
			out.write("</table>");

		} 
		catch (Exception e) 
		{
			e.printStackTrace(System.out);
		}
	}

	/* ---------------------------------------------------------- */
    
	public String getFullName(String inName)
	{
		if (!inName.startsWith(myBase)) 
		{
			return myBase + inName;
		}
		return inName;
	}

	/* The full string of a property is something like
	 * "theProperty:http://www.w3.org/2001/XMLSchema#string". 
	 * This function returns the first part "theProperty".
	 */ 
	public String getFirstPart(String inString)
	{
		int colonAt = inString.lastIndexOf(":http");
		return (inString.substring(0, colonAt));
	}  
    
	/* LongName will be "http://a.com/ontology#shortName"
	 * return the shortName out of the longName
	 */
	public String getShortName(String inName)
	{
		int pAt = inName.indexOf("#");
		return (inName.substring(pAt+1));
	}
    
	public OntModel loadModel()
	{
		OntModel theModel = ModelFactory.createOntologyModel();

		InputStream in = DisplayNEES.class.getClassLoader()
			.getResourceAsStream(inputFileName);
		if (in == null) 
		{
			throw new IllegalArgumentException(
				"File: " + inputFileName + " not found");
		}
        
		// read the OWL file
		theModel.read(in, "");
		return theModel;

	}
    

	/* Find the slot value of a particular instance.  
	 * The returned Vector contains RDFNode elements.
	 */
	public Vector getInstanceProperty(OntModel theModel, String theInstance, String theSlot) 
	{
		Vector theOutput = new Vector();
	
		theInstance = getFullName(theInstance);
		theSlot = getFullName(theSlot);

		Individual theObject = theModel.getIndividual(theInstance);
		// If the object doesn't exist, return an empty Vector.
		if (theObject == null) 
		{
			return theOutput;
		}

		// go through all the properties to find the relevent ones.
		StmtIterator sIter = theObject.listProperties();
		while (sIter.hasNext()) 
		{
			Statement stmt = sIter.nextStatement();
			
			Property predicate = stmt.getPredicate();	
			if (predicate.toString().startsWith(theSlot)) 
			{
				RDFNode oneResult = stmt.getObject();
				theOutput.add(oneResult);
			}
		}
		
		return theOutput;
	}
    
    
	public String getPropertyType(OntModel theModel, Resource oneProperty)
	{
		String theOutput = new String();

		StmtIterator sIter = oneProperty.listProperties();
		while (sIter.hasNext()) 
		{
			Statement stmt = sIter.nextStatement();
			Property predicate = stmt.getPredicate();

			if (predicate.getLocalName().startsWith("range")) 
			{
				RDFNode object = stmt.getObject();
				theOutput = object.toString();
			}
		}

		return theOutput;
	}
    

	/* Return all the slots of a certain class.
	 * The returned Vector contains Property elements.
	 */
	public Vector getClassSlots(OntModel theModel, String theClass)
	{
		Vector theOutput = new Vector();

		theClass = getFullName(theClass);
		OntClass actualClass = theModel.getOntClass(theClass);

		if (actualClass == null) 
		{
			return theOutput;
		}

		Iterator ppIter = actualClass.listDeclaredProperties(true);
		while (ppIter.hasNext()) 
		{
			Property oneProperty = (Property)ppIter.next();
			theOutput.add(oneProperty);
		}

		return theOutput;
	}    
    

	public Vector getInstanceAllProperties(OntModel theModel, Individual theObject)
	{
		String theInstance = theObject.getLocalName();
		return getInstanceAllProperties(theModel, theInstance);
	}

	/* Retrieve all the properties and their values.
	 * Each element in the Vector contains:  
	 *    String name;
	 *	  String type;
	 *    Vector allValues; (RDFNode)
	 */ 
	public Vector getInstanceAllProperties(OntModel theModel, String theInstance)
	{
		Vector theOutput = new Vector();
		theInstance = getFullName(theInstance);
		Individual theObject = theModel.getIndividual(theInstance);
		// If the object doesn't exist, return an empty Vector.
		if (theObject == null) 
		{
			return theOutput;
		}

		Resource theClass = theObject.getRDFType(true);
		Vector slots = getClassSlots(theModel, theClass.toString());

		for (int i = 0; i < slots.size(); i++) 
		{
			OneProperty oneEle = new OneProperty();
			Property oneP = (Property)slots.elementAt(i);
			String propertyName = oneP.getLocalName();
			String pType = getPropertyType(theModel, oneP); 

			Vector pros = getInstanceProperty(theModel, theInstance, propertyName);

			oneEle.name = propertyName;
			oneEle.type = pType;
			oneEle.allValues = pros;

			theOutput.add(oneEle);
		}

		return theOutput;
	}    
 
 
	public void listAllProjects(OntModel theModel, PrintWriter out)
	{
		OntClass theProject = theModel.getOntClass(myBase + "Project");
        
		out.println("<b>The system currently has the following Projects:</b>");
        
		for (Iterator i = theProject.listInstances(); i.hasNext(); ) 
		{
			Individual inst = (Individual) i.next();
			String projName = getShortName(inst.toString());
            
			out.println("<form method=\"POST\">\n" +
				"<input type=submit name=\"neesobject\" value=\"" + projName + "\">\n" +
				"</form>");
		}
	}
    

    
	public void displayAnInstance(OntModel theModel, Resource theObject, PrintWriter out)
	{
		String theInstance = theObject.getLocalName();
		displayAnInstance(theModel, theInstance, out);
	}

	/* Display all the properties of an Instance (object).
	 * Output to a PrintWriter and pipe to an webpage.
	 */  
	public void displayAnInstance(OntModel theModel, String theInstance, PrintWriter out) 
	{
		out.println("<b>The following is the details of " + theInstance + ":</b> <p>");
		Vector allResults = getInstanceAllProperties(theModel, theInstance);

		if (allResults.size() == 0) 
		{
			out.println("The object " + theInstance + " doesn't exist!");
		}

		out.println("<table border=1>");
		out.println("<tr> <td><b>SLOT</b></td>\n" +
			"     <td><b>VALUE</b></td> </tr>");
        
		for (int i = 0; i < allResults.size(); i++)
		{
			OneProperty oneEle = (OneProperty)allResults.elementAt(i);
			out.println("<tr> <td>" + oneEle.name + "</td>");
		
			Vector allValues = oneEle.allValues;
			out.println(" <td>");
			for (int jj = 0; jj < allValues.size(); jj++) 
			{
				if (oneEle.name.startsWith("URI")) 
				{
					String fileLocation = getFirstPart(allValues.elementAt(jj).toString());
					out.println("<a href=\"" + fileBase + fileLocation + "\">" + fileLocation + "</a>");
				}
				else if (oneEle.type.endsWith("string")) 
				{
					out.println(getFirstPart(allValues.elementAt(jj).toString()) + "<br>");
				}
				else if (oneEle.type.startsWith(myBase)) 
				{
					String objName = getShortName(allValues.elementAt(jj).toString());
					out.println("<form method=\"POST\">\n" +
						"<input type=submit name=\"neesobject\" value=\"" + objName + "\">\n" +
						"</form>");
				}
				else 
				{
					out.println(getFirstPart(allValues.elementAt(jj).toString()) + "<br>");
				}
			}
			out.println("</td> </tr>");
		}
		out.println("</table>");
	}
    



}